package org.harrisburgjug.example;

/**
 * Template object providing the structure of the DemoCommand's
 * execute() method and delegating to subclasses for customizable
 * functionality.
 * @author john
 */
public abstract class DemoTemplate implements DemoCommand {
	/**
	 * Business method to implement DemoCommand.
	 */
	public final void execute() {
		//  get the title from the subclass
		String title = getTitle();
		
		printHeader(title);
		
		// get the list from the subclass
		String[] list = getList();
		
		printList(list);
		
		printFooter(title);
	}
	
	/**
	 * Subclass must override this to provide title.
	 * @return title string
	 */
	public abstract String getTitle();
	
	/**
	 * Subclass must override this to provide array of strings.
	 * @return array of strings to be displayed
	 */
	public abstract String[] getList();
	
	/**
	 * Print an indented line to System.out.
	 * @param s String to print.
	 */
	public static void println(String s) {
		System.out.print("    ");
		System.out.println(s);
	}
	
	/**
	 * Generate a separator line with given length using the lineChar
	 * @param lineChar char to use for line
	 * @param length number of characters in line
	 * @return separator line string
	 */
	private static String separator(char lineChar, int length) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < length; i ++) {
			sb.append(lineChar);
		}
		return sb.toString();
	}

	/**
	 * Print formatted header with title.
	 * @param title title string.
	 */
	public static void printHeader(String title) {
		println(title);
		println(separator('-', title.length()));
	}
	
	/**
	 * Print formatted list of strings.
	 * @param s String array
	 */
	public static void printList(String[] s) {
		for (int i = 0; i < s.length; i ++) {
			StringBuffer sb = new StringBuffer();
			sb.append(i + 1);
			sb.append(". ");
			sb.append(s[i]);
			
			println(sb.toString());
		}
	}
	
	/**
	 * Print formatted footer with length to match title.
	 * @param title title used to determine length.
	 */
	public static void printFooter(String title) {
		println(separator('=', title.length()));
	}
}